#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    资源工厂
#    用于读取和解析sql文件资源
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from promise import Promise
from .screws import Store
from .loader import SqlParser
from types import MethodType, FunctionType
from .tool import FormatMsg
from .template_factory import templatePaser  # 模板转换器

# 根据外部传入的转换器，解析sql字符串，如果未传转换器，则直接读取sql字符串


class sqlLoad():
    def __init__(self, sqlPath, transFunc=None):
        self.sqlPath = sqlPath
        self.__sqlChips__ = None
        # sql转换方法
        self.transFunc = transFunc

    def __enter__(self):
        with open(self.sqlPath, "r", encoding='utf-8') as fs_sql:
            sql = fs_sql.read()
            if type(self.transFunc) == FunctionType:
                self.__sqlChips__ = self.transFunc(sql)
            else:
                self.__sqlChips__ = sql
        return self

    def read(self):
        return self.__sqlChips__

    def __exit__(self, exc_type, exc_value, traceback):
        return self


class ResourceFactory():
    def __init__(self):
        # sql转换器
        self.sqlParser = SqlParser()

    def sqlCompose(self, args=None, parseType=None, modelFnName=None, sql=None, logging=None, parserOptions={}):
        if logging == None:
            logging = self.sqlParser.logging
        # 读取sql字符串
        sqlStr = None
        sqlChips = templatePaser.Model(sql)
        if modelFnName == None:
            sqlStr = sql
        else:
            if sqlChips[modelFnName] == None:
                sqlStr = sql
            else:
                sqlStr = sqlChips[modelFnName]
        # 解析sql动作
        sqlAction = self.sqlParser.getSqlAction(sqlStr)
        # 插入的时候，可以一次插入多条数据
        if type(args) == list:
            if sqlAction == 'INSERT' or sqlAction == 'UPDATE' or sqlAction == 'DELETE':
                sqlStore = self.sqlParser.multiSqlParse(
                    sqlStr, args, logging, parseType=parseType, options=parserOptions)
                return sqlStore.transSqls, sqlAction
            else:
                errMsg = FormatMsg(
                    "When the '@sql' annotation function returns the list type, the sql statement must be a simple 'insert' or 'update' or 'delete'")
                logging('ERROR', errMsg)
                raise Exception(errMsg)
        else:
            singleSql = self.sqlParser.sqlParse(
                sqlStr, args, logging, parseType=parseType,quotation=parserOptions.get('quotation'))
            return singleSql, sqlAction

    def sqlLoad(self, sqlPath):
        sqlStr = None
        with sqlLoad(sqlPath) as res:
            sqlStr = res.read()
        return sqlStr


resf = ResourceFactory()




# 字符串true和false转bool型的True和False
def Str2Bool(boolVar):
    if type(boolVar) == str:
        if boolVar.lower() == 'true':
            return True
        elif boolVar.lower() == 'false':
            return False
        else:
            return bool(boolVar)
    else:
        return bool(boolVar)

# bool型的True和False转字符串的True和False


def Bool2Str(boolVar):
    if bool(boolVar):
        return "True"
    else:
        return "False"

# 字符串转整型


def Str2Int(intVar):
    return int(intVar)

# 整型转字符串


def Int2Str(intVar):
    return str(int(intVar))

# 获取第一个不为空的变量


def VarGet(*args):
    for arg in args:
        if arg != None:
            return arg
    return None

# 日志格式化输出


def FormatMsg(hmsg, *msgs):
    __msg = ""
    if len(msgs) > 0:
        __msg = f'>>> {hmsg} >>>'
    else:
        __msg = f'>>> {hmsg}'
    for msg in msgs:
        __msg = f'{__msg}\n{msg}'
    return __msg

# 段落字符串去每行前后空格


def TrimParagraph(pStr):
    fr = pStr.split('\n')
    trimStr = '\n'.join(list(map(lambda s: s.strip(), fr)))
    return trimStr



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    输出控制台模块
#    用于日志打印和控制台输出、错误追踪
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from __future__ import print_function
import logging
import logging.handlers
import chalk
from chalk import Chalk
import os
import math
import inspect
import traceback
import sys
import os
# 函数重载
from functools import singledispatch

# cmd中需要关闭echo
os.system('@echo off')
#

class LoggerFactory():
    def __init__(self, path='./default.log', console=False, debug=True, maxMb=1, backupCount=5, logOn=True):
        self.path = path
        self.console = console
        self.maxMb = maxMb
        self.backupCount = backupCount
        self.debug = debug
        self.logOn = logOn
        self.__create()

    # 初始化logging
    def __create(self, debug=None):
        # 无日志设置
        if self.logOn == False:
            self.__logger = None
            return
        # 路径截取的文件名做为模块名
        module_name = os.path.basename(self.path)
        # 获取logging对象
        self.__logger = logging.getLogger(module_name)
        #
        __debug = self.debug if debug == None else debug
        # 设置阈值 ，低于阈值以下的则不输出(DEBUG < INFO < WARNING < ERROR)
        if __debug == True:
            self.__logger.setLevel(logging.DEBUG)
        else:
            self.__logger.setLevel(logging.INFO)
        # 设置日志格式化 (时间 - 消息)
        __formatter = logging.Formatter('%(asctime)s - %(message)s', style='%')
        # log文件不存在的情况，初始化log
        if not self.__logger.handlers:
            if self.console:
                # 控制台输出的 handler
                __stream_handler = logging.StreamHandler()
                __stream_handler.setFormatter(__formatter)
                self.__logger.addHandler(__stream_handler)
            # 文件输出的 handler
            __file_handler = logging.handlers.RotatingFileHandler(
                filename=os.path.abspath(self.path),
                maxBytes=math.pow(2, 20)*self.maxMb,
                backupCount=self.backupCount,
                encoding='utf-8',
                delay=False)
            __file_handler.setFormatter(__formatter)
            self.__logger.addHandler(__file_handler)

    # 获取logger
    def logger(self, debug=True):
        self.__create(debug)
        return self.__logger

    # log输出
    def logging(self, level, message, cur_frame=None):
        # 无日志设置
        if self.logOn == False:
            self.__logger = None
            return
        # 获取当前帧对象 ， 代表执行到当前的logging函数
        cur_frame = cur_frame or inspect.currentframe()
        # 获取上一帧对象 ， 代表谁调用的logging
        bac_frame = cur_frame.f_back
        # 从帧对象中获取信息
        bac_frame_info = inspect.getframeinfo(bac_frame)
        # 截取文件名
        fileName = bac_frame_info.filename
        # 异常处理
        isError = isinstance(message, Exception)
        if isError:
            level = 'ERROR'
            message = traceback.format_exc()

        # 控制台打印异常信息
        if not self.console:
            self.print(level, message, cur_frame=bac_frame)

        message = "{0}[{1}] >>> {2:<6} >>> {3}".format(fileName,
                                                       str(bac_frame_info.lineno), level, message)

        if (level == "INFO"):
            self.__logger.info(message)
        elif (level == "ERROR"):
            self.__logger.error(message)
        elif (level == "DEBUG"):
            self.__logger.debug(message)
        elif (level == "WARN"):
            self.__logger.warning(message)

    def print(self, level, message, cur_frame=None):
        # 异常检测
        isError = isinstance(message, Exception)
        if isError:
            if level != 'ERROR' and level != 'WARN':
                level = 'ERROR'
            message = traceback.format_exc()
        # 获取当前帧对象 ， 代表执行到当前的logging函数
        # cur_frame = cur_frame or inspect.currentframe().f_back
        cur_frame = cur_frame or inspect.currentframe()
        # 获取上一帧对象 ， 代表谁调用的logging
        bac_frame = cur_frame.f_back
        # 从帧对象中获取信息
        bac_frame_info = inspect.getframeinfo(bac_frame)
        # 截取文件名
        fileName = bac_frame_info.filename
        if level == 'ERROR':
            # 控制台打印异常信息
            print(chalk.white('ERROR'.capitalize(), background='red'))
            print(chalk.red(message))
            self.print_traceback(color='red', cur_frame=bac_frame)
        elif level == 'WARN':
            # 控制台打印异常信息
            print(chalk.black('WARNING'.capitalize(), background='yellow'))
            print(chalk.yellow(message))
            if isError:
                self.print_traceback(
                    color='yellow', font_color='black', cur_frame=bac_frame)
        elif level == 'DEBUG':
            print(chalk.white(level.capitalize(), background='cyan'))
            print(chalk.cyan(message))
        else:
            print(message)

    # 追踪执行的文件、行号以及函数名
    def traceback(self, cur_frame=None):
        # 项目入口的根目录
        entry_dir = os.path.realpath('.')
        # 追踪记录
        trace_stack = []
        #
        cur_frame = cur_frame or inspect.currentframe().f_back
        # 上一帧
        pre_frame = None
        #
        isSkip = False
        while not isSkip:
            # 获取帧信息
            frame_info = inspect.getframeinfo(cur_frame)
            # 获取当前帧的目录
            if frame_info.function == '<module>':
                curr_dir = frame_info.filename
            else:
                curr_dir = os.path.dirname(frame_info.filename)
            # 记录文件名和行号
            trace_stack.append(
                [frame_info.filename, frame_info.lineno, frame_info.function])
            # 跳出
            if entry_dir == curr_dir or frame_info.function == '<module>':
                isSkip = True
            else:
                # 记录当前帧
                pre_frame = cur_frame
                # 向前追溯
                cur_frame = cur_frame.f_back
        return trace_stack

    # 打印追踪信息
    def print_traceback(self, color='red', font_color='white', cur_frame=None):
        # 获取当前帧对象
        cur_frame = cur_frame or inspect.currentframe().f_back
        # 获取追踪记录
        trace_stack = self.traceback(cur_frame)
        # 前一条记录
        pre_record = [None, None, None]
        # 层级列表
        tiers = []
        # 竖线
        verticalLine = chalk.white("", background='red')
        title = Chalk(font_color)("Exception", background=color)
        print(f'{verticalLine}{title}')
        for index in range(len(trace_stack)):
            record = trace_stack[index]
            if index == 0:
                content = Chalk(color)(
                    f" - {record[0]}({record[1]}) -> {record[2]}")
                print(f'{verticalLine}{content}')
            else:
                # 不在同一个文件中
                if pre_record[0] != record[0]:
                    if index > 1:
                        # 缩进
                        tiers.append('')
                    content = Chalk(color)(
                        f'{"".join(tiers)} - {record[0]}({record[1]}) -> {record[2]}')
                    print(f'{verticalLine}{content}')
                else:
                    content = Chalk(color)(
                        f'{"".join(tiers)}   {record[0]}({record[1]}) -> {record[2]}')
                    print(f'{verticalLine}{content}')
            # 记录上一个
            pre_record = record



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    数据库引导层
#    SQL挂载注解、DB连接注解
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from __future__ import absolute_import
from .frame_guide import InitGuide
from .bean_factory import bf
from .orm_factory import orm
from .resource_factory import resf
from .tool import FormatMsg, VarGet
from .screws import Store


def createDbc(*args, **kwargs):
    # 采用的数据库关系映射的库
    ORM_LOADER = kwargs.get('ORM_LOADER') or 'sqlalchemy'
    # 读取关系映射的loader
    OrmLoader: type = ORM_LOADER if type(
        ORM_LOADER) == type else orm.loaderCreater(ORM_LOADER)

    class Dbc(InitGuide, OrmLoader):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.__module_name__ = self.module_name
            # 绑定装饰器别名
            self.sql = self.__sql__
            self.conn = self.__connection__
            self.entry = self.__entry__
            # 初始化connection数据库连接对象
            self.connection = None

        # sql装饰器
        def __sql__(self, *args, **kwargs):
            ''' 装饰器参数 '''
            # 注入的Bean类，USE_BEAN=True才生效
            __bean__ = None if len(args) == 0 else args[0]
            argsDict = {
                # 直接读取sql文件，不从缓存中拿去
                'HARD_LOAD_SQL': VarGet(kwargs.get('HARD_LOAD_SQL'), self.HARD_LOAD_SQL),
                # 应用模板类型
                'SQL_TEMPLATE_TYPE': VarGet(kwargs.get('SQL_TEMPLATE_TYPE'), self.SQL_TEMPLATE_TYPE),
                # 是否应用Bean来使用自动注入
                'USE_BEAN': self.USE_BEAN,
                # 注入的Bean类
                'BEAN': __bean__,
                # 直接指定sql文件
                'SQL_NAME': kwargs.get('SQL_NAME'),
                # 直接设置sql语句
                'SQL': kwargs.get('SQL'),
                # 可识别最大sql长度
                'MAX_SQL_SIZE': self.MAX_SQL_SIZE,
                # 默认引号类型
                'QUOTATION':VarGet(kwargs.get('QUOTATION'), self.QUOTATION)
            }

            def _sql_(model_fn):
                # 解析路径
                # self.resolvePath(sql_on=argsDict['SQL_NAME']==None and argsDict['SQL']==None)
                # 初始化log
                self.initLogging()
                # 获取sql路径
                sql_path = None
                # 设置sql段落名，优先级： SQL参数 > SQL_NAME参数 > 自动查找
                modelFnName = None
                if argsDict['SQL'] == None:
                    if argsDict['SQL_NAME'] == None:
                        sql_path = self.resolveSqlPath(
                            model_fn.__name__, self.getModelPath(model_fn))
                        modelFnName = None
                    else:
                        sql_path = self.resolveSqlPathSn(argsDict['SQL_NAME'])
                        modelFnName = model_fn.__name__
                else:
                    modelFnName = model_fn.__name__

                def __model_fn(*args, **kwargs):
                    self.logging("DEBUG", FormatMsg(
                        "%s Start" % self.__module_name__))
                    # 如果参数传SQL，直接用参数的sql语句
                    if argsDict['SQL'] == None:
                        if argsDict['SQL_NAME'] == None:
                            # 每次都重新读取sql文件
                            if argsDict['HARD_LOAD_SQL']:
                                argsDict['SQL'] = resf.sqlLoad(sql_path)
                            else:
                                # 获取缓存中的sql字符串
                                if self.cache.all_sqls_cached:
                                    argsDict['SQL'] = self.cache.sql_str_dict.get(
                                        sql_path)
                                else:
                                    # 重新读取文件，并写入缓存
                                    argsDict['SQL'] = resf.sqlLoad(sql_path)
                                    self.cache.sql_str_dict[sql_path] = argsDict['SQL']
                        else:
                            argsDict['SQL'] = resf.sqlLoad(sql_path)

                    # 生成sql语句
                    sqlStr, sqlAction = resf.sqlCompose(
                        args=model_fn(*args, **kwargs),
                        parseType=argsDict['SQL_TEMPLATE_TYPE'],
                        modelFnName=modelFnName,
                        sql=argsDict['SQL'],
                        logging=self.logging,
                        parserOptions={
                            'maxSize':argsDict['MAX_SQL_SIZE'],
                            'quotation':argsDict['QUOTATION']
                        }
                    )

                    cursor = None
                    if self.RW_EXECUTE:
                        cursor = self.RW_EXECUTE(
                            self.connection, sqlStr, sqlAction)
                    else:
                        cursor = self.execute(
                            self.connection, sqlStr, sqlAction)

                    if argsDict['USE_BEAN']:
                        if argsDict['BEAN'] == None:
                            return cursor
                        else:
                            if self.RW_INJECT:
                                return self.RW_INJECT(self.connection, sqlAction, cursor, argsDict['BEAN'], bf.createResultClass)
                            else:
                                return self.inject(self.connection, sqlAction, cursor, argsDict['BEAN'], bf.createResultClass)
                    else:
                        return cursor
                return __model_fn
            return _sql_

        # 链接装饰器
        def __connection__(self, *args, **kwargs):
            # 接受缓存数据库的model_name
            _db_model_name_ = None if len(args) == 0 else args[0]
            # 防止闭包引用被垃圾回收
            argsDict = {
                'MODEL_NAME': _db_model_name_,
                'DB_CONFIG': self.getAccessInfo(_db_model_name_),
                'ALLOW_ROLLBACK': VarGet(kwargs.get('ALLOW_ROLLBACK'), self.ALLOW_ROLLBACK),
                'AUTO_COMMIT': VarGet(kwargs.get('AUTO_COMMIT'), self.AUTO_COMMIT)
            }

            def _connection_(logic_fn):
                # 解析路径
                # self.resolvePath()
                # 初始化log
                self.initLogging()
                #

                def __logic_fn(*args, **kwargs):
                    # 校验缓存的数据库连接
                    if self.CACHE_CONNECT and self.conn_cache.get(argsDict['MODEL_NAME']) == None:
                        raise Exception(FormatMsg("%s >> %s >> End" % (
                            argsDict['MODEL_NAME'], '@conn can`t get db connection.')))
                    #
                    if self.CACHE_CONNECT:
                        argsDict['ALLOW_ROLLBACK'] = VarGet(argsDict['DB_CONFIG'].get(
                            'ALLOW_ROLLBACK'), self.ALLOW_ROLLBACK)
                        argsDict['AUTO_COMMIT'] = VarGet(argsDict['DB_CONFIG'].get(
                            'AUTO_COMMIT'), self.AUTO_COMMIT)
                        # 获取连接
                        self.connection = self.conn_cache.get(
                            argsDict['MODEL_NAME'])
                    else:
                        dbConf = Store({
                            'DB_DRIVER': VarGet(self.DB_DRIVER, argsDict['DB_CONFIG'].get('DB_DRIVER')),
                            'DB_USER': VarGet(self.DB_USER, argsDict['DB_CONFIG'].get('DB_USER')),
                            'DB_PASSWORD': VarGet(self.DB_PASSWORD, argsDict['DB_CONFIG'].get('DB_PASSWORD')),
                            'DB_HOST': VarGet(self.DB_HOST, argsDict['DB_CONFIG'].get('DB_HOST')),
                            'DB_PORT': VarGet(self.DB_PORT, argsDict['DB_CONFIG'].get('DB_PORT')),
                            'DB_DATABASE': VarGet(self.DB_DATABASE, argsDict['DB_CONFIG'].get('DB_DATABASE')),
                            'ENCODING': VarGet(self.ENCODING, argsDict['DB_CONFIG'].get('ENCODING')),
                            'ALLOW_ROLLBACK': VarGet(argsDict['ALLOW_ROLLBACK'], argsDict['DB_CONFIG'].get('ALLOW_ROLLBACK')),
                            'AUTO_COMMIT': VarGet(argsDict['AUTO_COMMIT'], argsDict['DB_CONFIG'].get('AUTO_COMMIT')),
                            'SQLALCHEMY_ARGS': VarGet(self.SQLALCHEMY_ARGS)
                        })
                        if self.RW_CONNECT:
                            self.connection = self.RW_CONNECT(dbConf=dbConf)
                        else:
                            self.connection = self.connect(dbConf=dbConf)

                    self.logging('DEBUG', FormatMsg('DB Connection', '\n'.join((f'''
                        DB_DATABASE:{self.DB_DATABASE}
                        DB_USER:{self.DB_USER}
                        DB_PASSWORD:{self.DB_PASSWORD}
                        DB_HOST:{self.DB_HOST}
                        DB_PORT:{self.DB_PORT}
                    ''').split())))
                    result = None
                    try:
                        result = logic_fn(*args, **kwargs)
                        if argsDict['ALLOW_ROLLBACK'] and argsDict['AUTO_COMMIT']:
                            if self.RW_COMMIT:
                                self.RW_COMMIT(self.connection)
                            else:
                                self.commit(self.connection)
                    except Exception as exc:
                        if argsDict['ALLOW_ROLLBACK'] and argsDict['AUTO_COMMIT']:
                            if self.RW_ROLLBACK:
                                self.RW_ROLLBACK(self.connection)
                            else:
                                self.rollback(self.connection)
                            self.logging("ERROR", exc)
                            self.logging("WARN", FormatMsg("SQL Rollback"))
                            # 如果保持连接状态，数据库不关闭
                            if self.CACHE_CONNECT:
                                pass
                            else:
                                if self.RW_CLOSE:
                                    self.RW_CLOSE(self.connection)
                                else:
                                    self.close(self.connection)
                        raise exc
                    # 如果保持连接状态，数据库不关闭
                    if self.CACHE_CONNECT:
                        pass
                    else:
                        if self.RW_CLOSE:
                            self.RW_CLOSE(self.connection)
                        else:
                            self.close(self.connection)
                    self.logging("DEBUG", FormatMsg(
                        "%s End" % self.__module_name__))
                    return result
                return __logic_fn
            return _connection_

        # 主方法注解，可选注解，用于初始化
        def __entry__(self, *args, **kwargs):
            # @entry注解计数器
            self.cache.modify('entry_cnt', self.cache.entry_cnt + 1)
            if self.cache.entry_cnt > 1:
                raise Exception(FormatMsg("%s >> %s >> End" % (
                    self.__module_name__, '@entry annotation must be unique.')))
            # 解析sql,log,env文件夹路径
            # self.resolvePath()
            # 向导生成文件夹以及文件结构
            _StartGuide_ = kwargs.get('START_GUIDE')
            if _StartGuide_:
                self.runGuide()

            # 全局 HARD_LOAD_SQL 生效
            if not self.HARD_LOAD_SQL:
                # 是否缓存所有sqls数据
                _CacheSqls_ = kwargs.get('CACHE_SQLS')
                if _CacheSqls_ == True:
                    self.cacheSqlFiles()

            # 初始化log
            self.initLogging()

            # 缓存数据库连接
            if self.CACHE_CONNECT:
                self.cacheDbConn()

            def _entry_(logic_fn):
                # 解析路径
                # self.resolvePath()
                # 主方法做为根目录解析路径
                # 初始化log
                self.initLogging()

                def __entry_fn(*args, **kwargs):
                    return logic_fn(*args, **kwargs)
                return __entry_fn
            return _entry_

    # 实例化数据库向导
    return Dbc(*args, **kwargs)



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    框架引导层
#    全局参数定义、配置参数定义、默认参数生成、路径参数生成
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from __future__ import absolute_import
from types import MethodType, FunctionType
import sys
import os
import inspect
import traceback
import configparser
import logging
import logging.handlers
from dataclasses import dataclass
from .screws import PathPlant, Store, ConfStore
from .tool import Str2Bool, Str2Int, VarGet, FormatMsg
from .definition import __cache__
from .logger_factory import LoggerFactory


# conf文件获取初始化参数
class ConfArgs():
    def __init__(self, *args, **kwargs):
        # 初始化配置
        self.__configparser__ = configparser.ConfigParser()

        # 使用多环境切换
        self.__ENV_ON = kwargs.get("ENV_ON") or __cache__.env_on
        # 环境类型，用于多个环境切换
        self.__ENV_TYPE = kwargs.get("ENV_TYPE") or __cache__.env_type
        # 配置文件夹
        self.__ENV_DIR = PathPlant.transAbspath(
            kwargs.get('ENV_DIR') or __cache__.env_dir)
        if self.__ENV_ON:
            self.__CONF_PATH = os.path.join(
                self.__ENV_DIR, f'{self.__ENV_TYPE}.ini')
        else:
            # 配置文件路径
            self.__CONF_PATH = PathPlant.transAbspath(
                kwargs.get('CONF_PATH') or None)
        # 读取配置
        self.loadConf()

    def loadConf(self):
        if self.__CONF_PATH != None:
            self.__configparser__.read(self.__CONF_PATH, encoding="utf-8")
        return self.__configparser__

    def getArg(self, arg, namespace=__cache__.model_name):
        sec = self.__configparser__._sections
        if self.__configparser__.has_section(namespace):
            conf = sec.get(namespace)
            return conf.get(arg.lower())
        else:
            return None

    def getArgs(self, namespace=__cache__.model_name):
        sec = self.__configparser__._sections
        if self.__configparser__.has_section(namespace):
            args = sec.get(namespace)
            if args:
                return dict(args)
            else:
                return {}
        else:
            return {}

    def getArgsBySuffix(self, suffix, arg):
        namespace = f'{__cache__.conf_section_prefix}_{suffix}'
        return self.getArg(arg, namespace=namespace)

    def getSections(self, prefix=None):
        sec = self.__configparser__._sections
        secList = list(sec.keys())
        if prefix == None:
            return secList
        else:
            return list(filter(lambda x: (f'{prefix}_' in x) and x.index(f'{prefix}_') == 0, secList))

    # 配置文件夹
    @property
    def ENV_DIR(self):
        return self.__ENV_DIR

    @ENV_DIR.setter
    def ENV_DIR(self, value):
        self.__ENV_DIR = PathPlant.transAbspath(value)
        # pass  # 读写属性

    # 环境类型，用于多个环境切换
    @property
    def ENV_TYPE(self):
        return self.__ENV_TYPE

    @ENV_TYPE.setter
    def ENV_TYPE(self, value):
        self.__ENV_TYPE = value
        # pass  # 只读属性

    # 使用多环境切换
    @property
    def ENV_ON(self):
        return self.__ENV_ON

    @ENV_ON.setter
    def ENV_ON(self, value):
        self.__ENV_ON = value
        # pass  # 只读属性

# 初始化参数


class GuideArgs(ConfArgs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # print("GuideArgs", kwargs)
        # 创建参数字典
        self.__Args__ = dict()
        # SQL文件存放目录
        self.__Args__['SQL_PATH'] = PathPlant.transAbspath(kwargs.get(
            'SQL_PATH') or self.getArg('SQL_PATH') or None)
        # LOG文件存放目录
        self.__Args__['LOG_PATH'] = PathPlant.transAbspath(kwargs.get(
            'LOG_PATH') or self.getArg('LOG_PATH') or None)
        # 是否输出日志文件
        self.__Args__['LOG_ON'] = VarGet(kwargs.get(
            'LOG_ON'), self.getArg('LOG_ON'), __cache__.log_on)
        # 是否是debug模式
        self.__Args__['DEBUG'] = VarGet(kwargs.get(
            'DEBUG'), self.getArg('DEBUG'), __cache__.debug)
        self.__Args__['DEBUG'] = Str2Bool(self.__Args__['DEBUG'])
        # 单个日志文件最大容量(mb)
        self.__Args__['LOG_MAX_SIZE'] = VarGet(kwargs.get(
            'LOG_MAX_SIZE'), self.getArg('LOG_MAX_SIZE'), __cache__.log_max_size)
        self.__Args__['LOG_MAX_SIZE'] = Str2Int(self.__Args__['LOG_MAX_SIZE'])
        # 日志最大备份数
        self.__Args__['LOG_BACKUP_CNT'] = VarGet(kwargs.get(
            'LOG_BACKUP_CNT'), self.getArg('LOG_BACKUP_CNT'), __cache__.log_backup_cnt)
        self.__Args__['LOG_BACKUP_CNT'] = Str2Int(
            self.__Args__['LOG_BACKUP_CNT'])
        # 是否追踪子文件夹下的SQL文件
        self.__Args__['TRACK_SQL_FILE'] = VarGet(kwargs.get(
            'TRACK_SQL_FILE'), self.getArg('TRACK_SQL_FILE'), __cache__.track_sql_file)
        self.__Args__['TRACK_SQL_FILE'] = Str2Bool(
            self.__Args__['TRACK_SQL_FILE'])
        # 指定model文件夹名称，默认是model
        self.__Args__['MODEL_FOLDER_NAME'] = VarGet(kwargs.get(
            'MODEL_FOLDER_NAME'), self.getArg('MODEL_FOLDER_NAME'), __cache__.model_folder_name)

        # 是否使用Bean来获取数据
        self.__Args__['USE_BEAN'] = VarGet(kwargs.get(
            'USE_BEAN'), self.getArg('USE_BEAN'), __cache__.use_bean)
        # 是否在异常时自动回滚
        self.__Args__['ALLOW_ROLLBACK'] = VarGet(kwargs.get(
            "ALLOW_ROLLBACK"), self.getArg('ALLOW_ROLLBACK'), __cache__.allow_rollback)
        # 是否自动提交
        self.__Args__['AUTO_COMMIT'] = VarGet(kwargs.get(
            "AUTO_COMMIT"), self.getArg('AUTO_COMMIT'), __cache__.auto_commit)
        # SQL模板类型
        self.__Args__['SQL_TEMPLATE_TYPE'] = VarGet(kwargs.get(
            "SQL_TEMPLATE_TYPE"), self.getArg('SQL_TEMPLATE_TYPE'), __cache__.sql_template_type)
        # 可识别最大sql长度
        self.__Args__['MAX_SQL_SIZE'] = VarGet(kwargs.get(
            "MAX_SQL_SIZE"), self.getArg('MAX_SQL_SIZE'), __cache__.max_sql_size)
        # 默认引号类型
        self.__Args__['QUOTATION'] = VarGet(kwargs.get(
            "QUOTATION"), self.getArg('QUOTATION'), __cache__.quotation)

        # 数据库驱动
        self.__Args__['DB_DRIVER'] = VarGet(kwargs.get(
            "DB_DRIVER"), self.getArg('DB_DRIVER'), None)
        # 数据库认证
        self.__Args__['DB_DATABASE'] = VarGet(kwargs.get(
            "DB_DATABASE"), self.getArg('DB_DATABASE'), None)
        self.__Args__['DB_USER'] = VarGet(kwargs.get(
            "DB_USER"), self.getArg('DB_USER'), None)
        self.__Args__['DB_PASSWORD'] = VarGet(kwargs.get(
            "DB_PASSWORD"), self.getArg('DB_PASSWORD'), None)
        self.__Args__['DB_HOST'] = VarGet(kwargs.get(
            "DB_HOST"), self.getArg('DB_HOST'), __cache__.db_host)
        # 默认端口号设置
        __default_port = None
        if self.__Args__['DB_DRIVER'] == "postgres":
            __default_port = 5432
        elif self.__Args__['DB_DRIVER'] == "mysql":
            __default_port = 3306
        # 设置端口号
        self.__Args__['DB_PORT'] = VarGet(kwargs.get(
            "DB_PORT"), self.getArg('DB_PORT'), __default_port, None)
        # sqlite db文件路径
        self.__Args__['SQLITE_PATH'] = VarGet(kwargs.get(
            "SQLITE_PATH"), self.getArg('SQLITE_PATH'), None)

        # 数据库编码
        self.__Args__['ENCODING'] = VarGet(kwargs.get(
            "ENCODING"), self.getArg('ENCODING'), __cache__.encoding)

        # sqlalchemy扩展参数
        self.__Args__['SQLALCHEMY_ARGS'] = Store(VarGet(kwargs.get("SQLALCHEMY_ARGS") if type(
            kwargs.get("SQLALCHEMY_ARGS")) == dict else None, __cache__.sqlalchemy_args))

        # loader外部重载方法
        # connect()
        self.__Args__['RW_CONNECT'] = kwargs.get("RW_CONNECT") if type(
            kwargs.get("RW_CONNECT")) == FunctionType else None
        # execute()
        self.__Args__['RW_EXECUTE'] = kwargs.get("RW_EXECUTE") if type(
            kwargs.get("RW_EXECUTE")) == FunctionType else None
        # close()
        self.__Args__['RW_CLOSE'] = kwargs.get("RW_CLOSE") if type(
            kwargs.get("RW_CLOSE")) == FunctionType else None
        # commit()
        self.__Args__['RW_COMMIT'] = kwargs.get("RW_COMMIT") if type(
            kwargs.get("RW_COMMIT")) == FunctionType else None
        # rollback()
        self.__Args__['RW_ROLLBACK'] = kwargs.get("RW_ROLLBACK") if type(
            kwargs.get("RW_ROLLBACK")) == FunctionType else None
        # inject()
        self.__Args__['RW_INJECT'] = kwargs.get("RW_INJECT") if type(
            kwargs.get("RW_INJECT")) == FunctionType else None

        # 每次都重新读取sql文件，不进缓存
        self.__Args__['HARD_LOAD_SQL'] = VarGet(kwargs.get(
            "HARD_LOAD_SQL"), self.getArg('HARD_LOAD_SQL'), __cache__.hard_load_sql)

        # 缓存数据库连接，保持数据库连接对象，数据库关闭失效
        self.__Args__['CACHE_CONNECT'] = VarGet(kwargs.get(
            "CACHE_CONNECT"), self.getArg('CACHE_CONNECT'), __cache__.cache_connect)

        # 缓存DB预设
        self.__Args__['DB_CONF'] = VarGet(kwargs.get(
            "DB_CONF"), self.getArg('DB_CONF'), None)

    # 获取参数字典

    @property
    def _Args_(self):
        return self.__Args__

    # SQL文件存放目录
    @property
    def SQL_PATH(self):
        return self.__Args__['SQL_PATH']

    @SQL_PATH.setter
    def SQL_PATH(self, value):
        self.__Args__['SQL_PATH'] = PathPlant.transAbspath(value)
        # pass  # 可读写属性

    # LOG文件存放目录
    @property
    def LOG_PATH(self):
        return self.__Args__['LOG_PATH']

    @LOG_PATH.setter
    def LOG_PATH(self, value):
        self.__Args__['LOG_PATH'] = PathPlant.transAbspath(value)
        # pass  # 可读写属性

    # 是否输出日志文件
    @property
    def LOG_ON(self):
        return self.__Args__['LOG_ON']

    @LOG_ON.setter
    def LOG_ON(self, value):
        self.__Args__['LOG_ON'] = value
        # pass  # 可读写属性

    # 是否是debug模式
    @property
    def DEBUG(self):
        return self.__Args__['DEBUG']

    @DEBUG.setter
    def DEBUG(self, value):
        self.__Args__['DEBUG'] = value
        # pass  # 可读写属性

    # 单个日志文件最大容量(mb)
    @property
    def LOG_MAX_SIZE(self):
        return self.__Args__['LOG_MAX_SIZE']

    @LOG_MAX_SIZE.setter
    def LOG_MAX_SIZE(self, value):
        self.__Args__['LOG_MAX_SIZE'] = value
        # pass  # 可读写属性

    # 日志最大备份数
    @property
    def LOG_BACKUP_CNT(self):
        # print(self.__Args__)
        return self.__Args__['LOG_BACKUP_CNT']

    @LOG_BACKUP_CNT.setter
    def LOG_BACKUP_CNT(self, value):
        self.__Args__['LOG_BACKUP_CNT'] = value
        # pass  # 可读写属性

    # 是否追踪子文件夹下的SQL文件
    @property
    def TRACK_SQL_FILE(self):
        return self.__Args__['TRACK_SQL_FILE']

    @TRACK_SQL_FILE.setter
    def TRACK_SQL_FILE(self, value):
        # self.__Args__['TRACK_SQL_FILE'] = value
        pass  # 只读属性

    # 指定model文件夹名称，默认是model
    @property
    def MODEL_FOLDER_NAME(self):
        return self.__Args__['MODEL_FOLDER_NAME']

    @MODEL_FOLDER_NAME.setter
    def MODEL_FOLDER_NAME(self, value):
        # self.__Args__['MODEL_FOLDER_NAME'] = value
        pass  # 只读属性

    # 是否使用Bean来获取数据
    @property
    def USE_BEAN(self):
        return self.__Args__['USE_BEAN']

    @USE_BEAN.setter
    def USE_BEAN(self, value):
        # self.__Args__['USE_BEAN'] = value
        pass  # 只读属性

    # 是否在异常时自动回滚
    @property
    def ALLOW_ROLLBACK(self):
        return self.__Args__['ALLOW_ROLLBACK']

    @ALLOW_ROLLBACK.setter
    def ALLOW_ROLLBACK(self, value):
        # self.__Args__['ALLOW_ROLLBACK'] = value
        pass  # 只读属性

    # 是否自动提交
    @property
    def AUTO_COMMIT(self):
        return self.__Args__['AUTO_COMMIT']

    @AUTO_COMMIT.setter
    def AUTO_COMMIT(self, value):
        # self.__Args__['AUTO_COMMIT'] = value
        pass  # 只读属性

    # SQL模板类型
    @property
    def SQL_TEMPLATE_TYPE(self):
        return self.__Args__['SQL_TEMPLATE_TYPE']

    @SQL_TEMPLATE_TYPE.setter
    def SQL_TEMPLATE_TYPE(self, value):
        # self.__Args__['SQL_TEMPLATE_TYPE'] = value
        pass  # 只读属性

    # 可识别最大sql长度
    @property
    def MAX_SQL_SIZE(self):
        return self.__Args__['MAX_SQL_SIZE']

    @MAX_SQL_SIZE.setter
    def MAX_SQL_SIZE(self, value):
        # self.__Args__['MAX_SQL_SIZE'] = value
        pass  # 只读属性

    # 默认引号类型
    @property
    def QUOTATION(self):
        return self.__Args__['QUOTATION']

    @QUOTATION.setter
    def QUOTATION(self, value):
        # self.__Args__['QUOTATION'] = value
        pass  # 只读属性

    # 数据库驱动
    @property
    def DB_DRIVER(self):
        return self.__Args__['DB_DRIVER']

    @DB_DRIVER.setter
    def DB_DRIVER(self, value):
        self.__Args__['DB_DRIVER'] = value
        # pass  # 只读属性

    # 数据库认证
    @property
    def DB_DATABASE(self):
        return self.__Args__['DB_DATABASE']

    @DB_DATABASE.setter
    def DB_DATABASE(self, value):
        self.__Args__['DB_DATABASE'] = value
        # pass  # 只读属性

    @property
    def DB_USER(self):
        return self.__Args__['DB_USER']

    @DB_USER.setter
    def DB_USER(self, value):
        self.__Args__['DB_USER'] = value
        # pass  # 只读属性

    @property
    def DB_PASSWORD(self):
        return self.__Args__['DB_PASSWORD']

    @DB_PASSWORD.setter
    def DB_PASSWORD(self, value):
        self.__Args__['DB_PASSWORD'] = value
        # pass  # 只读属性

    @property
    def DB_HOST(self):
        return self.__Args__['DB_HOST']

    @DB_HOST.setter
    def DB_HOST(self, value):
        self.__Args__['DB_HOST'] = value
        # pass  # 只读属性

    @property
    def DB_PORT(self):
        return self.__Args__['DB_PORT']

    @DB_PORT.setter
    def DB_PORT(self, value):
        self.__Args__['DB_PORT'] = value
        # pass  # 只读属性

    # sqlite db文件路径
    @property
    def SQLITE_PATH(self):
        return self.__Args__['SQLITE_PATH']

    @SQLITE_PATH.setter
    def SQLITE_PATH(self, value):
        self.__Args__['SQLITE_PATH'] = value
        # pass  # 只读属性

    # 数据库编码
    @property
    def ENCODING(self):
        return self.__Args__['ENCODING']

    @ENCODING.setter
    def ENCODING(self, value):
        self.__Args__['ENCODING'] = value
        # pass  # 只读属性

    # sqlalchemy扩展参数
    @property
    def SQLALCHEMY_ARGS(self):
        return self.__Args__['SQLALCHEMY_ARGS']

    @SQLALCHEMY_ARGS.setter
    def SQLALCHEMY_ARGS(self, value):
        self.__Args__['SQLALCHEMY_ARGS'] = value
        # pass  # 只读属性

    # loader外部重载方法
    @property
    def RW_CONNECT(self):
        return self.__Args__['RW_CONNECT']

    @RW_CONNECT.setter
    def RW_CONNECT(self, value):
        # self.__Args__['RW_CONNECT'] = value
        pass  # 只读属性

    @property
    def RW_EXECUTE(self):
        return self.__Args__['RW_EXECUTE']

    @RW_EXECUTE.setter
    def RW_EXECUTE(self, value):
        # self.__Args__['RW_EXECUTE'] = value
        pass  # 只读属性

    @property
    def RW_CLOSE(self):
        return self.__Args__['RW_CLOSE']

    @RW_CLOSE.setter
    def RW_CLOSE(self, value):
        # self.__Args__['RW_CLOSE'] = value
        pass  # 只读属性

    @property
    def RW_COMMIT(self):
        return self.__Args__['RW_COMMIT']

    @RW_COMMIT.setter
    def RW_COMMIT(self, value):
        # self.__Args__['RW_COMMIT'] = value
        pass  # 只读属性

    @property
    def RW_ROLLBACK(self):
        return self.__Args__['RW_ROLLBACK']

    @RW_ROLLBACK.setter
    def RW_ROLLBACK(self, value):
        # self.__Args__['RW_ROLLBACK'] = value
        pass  # 只读属性

    @property
    def RW_INJECT(self):
        return self.__Args__['RW_INJECT']

    @RW_INJECT.setter
    def RW_INJECT(self, value):
        # self.__Args__['RW_INJECT'] = value
        pass  # 只读属性

    # 每次都重新读取sql文件，不进缓存
    @property
    def HARD_LOAD_SQL(self):
        return self.__Args__['HARD_LOAD_SQL']

    @HARD_LOAD_SQL.setter
    def HARD_LOAD_SQL(self, value):
        # self.__Args__['HARD_LOAD_SQL'] = value
        pass  # 只读属性

    # 缓存数据库连接，保持数据库连接对象，数据库关闭失效
    @property
    def CACHE_CONNECT(self):
        return self.__Args__['CACHE_CONNECT']

    @CACHE_CONNECT.setter
    def CACHE_CONNECT(self, value):
        # self.__Args__['CACHE_CONNECT'] = value
        pass  # 只读属性

    # 多DB预设
    @property
    def DB_CONF(self):
        return self.__Args__['DB_CONF']

    @DB_CONF.setter
    def DB_CONF(self, value):
        # self.__Args__['DB_CONF'] = value
        pass  # 只读属性


class InitGuide(GuideArgs, PathPlant):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # 模块名字
        self.module_name = __cache__.model_name if len(args) == 0 else args[0]
        # 生成缓存对象，一个实例生成一个缓存对象
        self.cache = ConfStore(
            # 已缓存全部sql数据
            all_sqls_cached=False,
            # @entry注解调用计数，目前只允许一个
            entry_cnt=0,
            # # 一次性读取的sql文件路径
            # folder_structure=list(),
            # # 一次性缓存的sql数据
            # sql_str_dict=dict(),
            # 缓存DB连接头
            # conn_headstr = list()
        )
        # 缓存DB连接的字典
        self.conn_cache = dict()
        # 解析sql,log,env文件夹路径
        self.resolvePath()

    # 从配置中获取DB信息
    def getDBInfoFromConf(self, dbModelName):
        return {
            'DB_DRIVER': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'DB_DRIVER'),
            'DB_DATABASE': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'DB_DATABASE'),
            'DB_USER': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'DB_USER'),
            'DB_PASSWORD': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'DB_PASSWORD'),
            'DB_HOST': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'DB_HOST'),
            'DB_PORT': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'DB_PORT'),
            'ALLOW_ROLLBACK': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'ALLOW_ROLLBACK'),
            'AUTO_COMMIT': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'AUTO_COMMIT'),
            'ENCODING': None if dbModelName == None else self.getArgsBySuffix(dbModelName, 'ENCODING'),
        }

    # 获取数据库连接信息
    def getAccessInfo(self, dbModelName):
        if self.DB_CONF == None:
            DB_CONF = self.getDBInfoFromConf(dbModelName)
        else:
            if self.DB_CONF.get(dbModelName) == None:
                DB_CONF = self.getDBInfoFromConf(dbModelName)
            else:
                DB_CONF = self.DB_CONF.get(dbModelName)
        return DB_CONF

    # 获取数据库连接头列表
    def getAccessHeadStr(self):
        # 从参数中获取
        conn_headstr = []
        for model_name in self.DB_CONF:
            conn_headstr.append(model_name)
        # 从配置文件中获取
        for model_name in self.getSections(__cache__.conf_section_prefix):
            conn_headstr.append(model_name)
        # 去重
        conn_headstr = list(set(conn_headstr))
        self.cache.create('conn_headstr', conn_headstr)
        return conn_headstr

    # 解析文件夹路径
    def resolvePath(self, sql_on=False):
        if sql_on:
            # 解决sql文件夹
            if self.SQL_PATH == None:
                self.SQL_PATH = os.path.realpath('sql')
            # self.initFolder(self.SQL_PATH)
            # 路径写入缓存
            __cache__.modify('sql_dir', self.SQL_PATH)
        # 解决log文件夹
        if self.LOG_ON:
            if self.LOG_PATH == None:
                self.LOG_PATH = os.path.realpath('log')
            # self.initFolder(self.LOG_PATH)
            # 路径写入缓存
            __cache__.modify('log_dir', self.LOG_PATH)
        # 解决env文件夹
        if self.ENV_ON:
            if self.ENV_DIR == None:
                self.ENV_DIR = os.path.realpath('env')
            # self.initFolder(self.ENV_DIR)
            # 路径写入缓存
            __cache__.modify('env_dir', self.ENV_DIR)

    # 解决sql后缀名
    def resolveSqlExtension(self, sqlname):
        return sqlname if sqlname[-4:] == '.sql' else sqlname + '.sql'

    # 根据sql文件名查找sql路径
    def getSqlPath(self, sqlname):
        # 解析后缀名
        _sqlname = self.resolveSqlExtension(sqlname)
        # 缓存sql文件列表
        if self.cache.folder_structure == None:
            self.cacheSqlPaths()
        #
        for sqlpath in self.cache.folder_structure.PATH_LIST:
            if os.path.basename(sqlpath) == _sqlname:
                return sqlpath
        return None

    # 缓存sql文件路径
    def cacheSqlPaths(self):
        if not self.cache.folder_structure:
            folder_structure = self.deepenFolder(self.SQL_PATH)
            self.cache.create('folder_structure', folder_structure)

    # 缓存sql字符串
    def cacheSqlString(self, sql_path_list):
        sql_str_dict = {}
        for sqlPath in sql_path_list:
            with open(sqlPath, "r", encoding='utf-8') as fs_sql:
                sqlStr = fs_sql.read()
                sql_str_dict[sqlPath] = sqlStr
        self.cache.create('sql_str_dict', sql_str_dict)

    # 解析sql文件路径
    def resolveSqlPath(self, func_name, model_path):
        if self.TRACK_SQL_FILE and self.MODEL_FOLDER_NAME != None:
            # 模块路径截取，model_path是包含文件名的
            preModelPath, relModelPath = self.splitFolder(
                model_path, self.MODEL_FOLDER_NAME, includeModel=False)
            # sql文件全路径,[:-3]是去掉后缀名.py
            sql_fullpath = os.path.join(
                self.SQL_PATH, relModelPath[:-3], func_name + '.sql')
            return sql_fullpath
        else:
            sql_fullpath = self.getSqlPath(func_name)
            return sql_fullpath

    # 通过sql_name解析sql文件路径
    def resolveSqlPathSn(self, sql_name):
        sql_fullpath = os.path.join(
            self.SQL_PATH, self.resolveSqlExtension(sql_name))
        return sql_fullpath

    # 主动缓存sql文件
    def cacheSqlFiles(self):
        # 读取全部sql文件
        self.cacheSqlPaths()
        # 读取全部的sql字符串
        self.cacheSqlString(self.cache.folder_structure.PATH_LIST)
        # 把已缓存sql标记为True
        self.cache.modify('all_sqls_cached', True)

    # 主动缓存db连接
    def cacheDbConn(self):
        for model_name in self.getAccessHeadStr():
            dbConf = self.getAccessInfo(model_name)
            if self.RW_CONNECT:
                self.conn_cache[model_name] = self.RW_CONNECT(dbConf=dbConf)
            else:
                self.conn_cache[model_name] = self.connect(dbConf=dbConf)

    # 主动调用路径模板向导
    def runGuide(self):
        self.initFolder(self.SQL_PATH)
        self.initFolder(self.LOG_PATH)
        self.initFolder(self.ENV_DIR)

    # 控制台输出
    def __print(self, level, message, *msgs):
        # 获取当前帧对象 ， 代表执行到当前的logging函数
        cur_frame = inspect.currentframe()
        # 获取上一帧对象 ， 代表谁调用的
        # bac_frame = cur_frame.f_back
        bac_frame = cur_frame
        # 字符串转换
        __msgs = tuple(map(lambda m: str(m), msgs))
        # 添加换行符
        msgStr = '\n'.join(__msgs)
        # 字符串message
        if type(message) == str:
            # 合并
            message = message + '\n' + msgStr
            self.__logger.print(level, message, cur_frame=bac_frame)
        else:
            # Exception message
            self.__logger.print(level, msgStr, cur_frame=bac_frame)
            self.__logger.print(level, message, cur_frame=bac_frame)
        # self.__logger.print(level, message, cur_frame=bac_frame)

    # 日志输出
    def __logging(self, level, message, *msgs):
        # 获取当前帧对象 ， 代表执行到当前的logging函数
        cur_frame = inspect.currentframe()
        # 获取上一帧对象 ， 代表谁调用的
        # bac_frame = cur_frame.f_back
        bac_frame = cur_frame
        # 字符串转换
        __msgs = tuple(map(lambda m: str(m), msgs))
        # 添加换行符
        msgStr = '\n'.join(__msgs)
        # 字符串message
        if type(message) == str:
            # 合并
            message = message + '\n' + msgStr
            self.__logger.logging(level, message, cur_frame=bac_frame)
        else:
            # Exception message
            self.__logger.logging(level, msgStr, cur_frame=bac_frame)
            self.__logger.logging(level, message, cur_frame=bac_frame)

    # 打印输出函数
    def logging(self, level, message, *msgs):
        self.__print(level, message, *msgs)
        if self.LOG_ON:
            self.__logging(level, message, *msgs)

    # 初始化logging
    def initLogging(self):
        self.__logger = LoggerFactory(
            path=os.path.join(self.LOG_PATH, self.module_name + ".log"),
            console=False,
            debug=self.DEBUG,
            maxMb=self.LOG_MAX_SIZE,
            backupCount=self.LOG_BACKUP_CNT,
            logOn=self.LOG_ON)



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    全局定义
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from .screws import ConfStore, Store
# 默认参数常量
__cache__ = ConfStore(
    # 默认路径设置
    # root_dir=None,  # 项目根目录
    # model_dir=None,  # 模型文件目录
    # conf_file=None,  # 配置文件路径
    sql_dir='sql',  # sql文件目录
    log_dir='log',  # log文件目录
    # 是否输出日志文件
    log_on=False,
    # 是否使用多环境切换
    env_on=False,
    # 是否是debug模式
    debug=True,
    # 单个日志文件最大容量(mb)
    log_max_size=10,
    # 日志最大备份数
    log_backup_cnt=1,
    # 是否追踪子文件夹下的SQL文件
    track_sql_file=False,
    # 默认模块文件夹名称
    model_folder_name='model',
    # 默认模块名
    model_name='straw',
    # 自动Bean注入
    use_bean=True,
    # 是否自动回滚
    allow_rollback=True,
    # 是否自动提交
    auto_commit=True,
    # sql模板类型
    sql_template_type=6,
    # 默认hostname
    db_host='localhost',
    # 数据库编码
    encoding='utf-8',
    # sqlalchemy扩展参数
    sqlalchemy_args={
        'create_engine':None,
        'sessionmaker':None,
        'scoped_session':None
    },
    # 每次都重新读取sql文件，不进缓存
    hard_load_sql=False,
    # 缓存数据库连接，保持数据库连接对象，数据库关闭失效
    cache_connect=False,
    # 预设db配置文件的section前缀
    conf_section_prefix='db',
    # 环境类型，用于多个环境切换
    env_type='dev',
    # 默认的env路径名
    env_dir = 'env',
    # 最大sql长度
    max_sql_size = 1024*512,
    # sql默认引号
    quotation = '\''

)

# DB类型反射python类型
__type_reflect__ = Store(
    {
        "postgres":{
            "varchar":str,
            "int":int,
            "datetime":str
        },
        "mysql":{
            "varchar":str,
            "int":int,
            "datetime":str
        }
    }
)

GlobalConfig = __cache__
SQLTypeMap = __type_reflect__



# import configparser
# import os
# pa = os.path.abspath('./conf.env')
# a = configparser.ConfigParser()
# a.read(pa, encoding="utf-8")
# sec = a._sections
# conf = sec.get('aaaa')
# print(conf)

def aaa(*args,**kwargs):
    x = True
    def _aaa_(_fnname):
        print(x)
        def fun(*args,**kwargs):
            print(x)
            return _fnname(*args,**kwargs)
        return fun
    return _aaa_

@aaa()
def bbb():
    return None

bbb()


#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    对象关系模型（ORM）工厂类
#    用于处理对象关系模型的Loader
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from __future__ import absolute_import
from . import loader


class OrmFactory():
    def __init__(self):
        super().__init__()

    # 对象关系模型loader构造器
    def loaderCreater(self, loader):
        exec(f'from .loader import {loader}')
        __module__ = eval(loader)
        __loader__ = __module__.loader

        class OrmEngine(__loader__):
            def __init__(self, *args, **kwargs):
                super().__init__()
        return OrmEngine


orm = OrmFactory()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .db_guide import createDbc as Straw
from .bean_factory import Bean
from .screws import Store, ConfStore
from .definition import GlobalConfig
from .resource_factory import resf as Transition
from .loader import OrmLoader


#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    ORM Loader 扩展模块
# Created : 2020.10.22
# Author : Chalk Yu
# ========================================
from __future__ import absolute_import
import sqlparse
import re
import math
from ..screws import Store
from ..tool import FormatMsg,TrimParagraph,VarGet
from ..definition import __cache__


# ORM Loader接口
class OrmLoader():

    def __init__(self):
        super().__init__()

    def logging(self, type, message):
        print(type, message)

    '''
        数据库连接：
        参数：
            [allow_rollback] : default = False
            [auto_commit] : default = True
        传入方式：
            @[dbc].connection([allow_rollback],[auto_commit])
    '''

    def connect(self, dbConf=None):
        return None

    '''
        sql执行：
        参数：
            [allow_rollback] : default = False
            [auto_commit] : default = True
        传入方式：
            @[dbc].connection([allow_rollback],[auto_commit])
    '''

    # def execute(self, sql_path, args, parseType, modelFnName):
    #     pass

    def execute(self, sqls, sqlAction=None):
        return [None]

    def close(self):
        return None

    def commit(self):
        return None

    def rollback(self):
        return None

    def inject(self, curs, bean, resultCreater):
        return None


# SQL转换类
class SqlParser():
    def __init__(self):
        super().__init__()

    # logging检测
    #   @args：
    #      [logging] : logging
    #   @return：logging
    def logger(self, logging):
        if(logging is None):
            return print
        else:
            return logging

    # sql格式化
    #   @args：
    #      [sql] : str(sql)
    #      [loc] : index of sql, when there are multi-sqls in str(sql)
    #   @return：str(sql)
    def formatSql(self, sql, loc=0):
        format_sql = sqlparse.format(
            sql, reindent=True, keyword_case='upper', strip_comments=True, reindent_aligned=False)
        parsed = sqlparse.parse(format_sql)
        stmt = parsed[loc]
        sql = str(stmt)
        sql = TrimParagraph(sql) # 清除缩进产生的前后空格
        return sql

    # list替换
    #   @args：
    #      [sourceList] : sourceList
    #      [oldVal] : search value
    #      [newVal] : replace value
    #   @return：targetList
    def listReplace(self, sourceList, oldVal, newVal):
        targetList = []
        for item in sourceList:
            if item.strip() == oldVal.strip():
                targetList.append(newVal)
            else:
                targetList.append(item)
        return targetList

    # list拆分（不均匀拆分）
    def listSplit(self, sourceList, maxSize):
        targetList = []
        subList = []
        loc = 0
        for item in sourceList:
            # 子集长度累加
            loc = loc + len(item)
            # 累加后的子集长度如果小于最大长度则添加
            if loc < maxSize:
                subList.append(item)
            # 累加后的子集长度如果大于最大长度则执行下一轮
            else:
                targetList.append(subList)
                # 初始化子集
                subList = []
                # 当前元素添加到新子集中
                subList.append(item)
                # 重新计算新子集长度
                loc = len(item)
        # 总长度都没超过maxSize
        if len(targetList) == 0:
            targetList.append(subList)
        #
        return targetList

    # sql碎片化
    #   @args：
    #      [sql] : str(sql)
    #      [loc] : index of sql, when there are multi-sqls in str(sql)
    #   @return：list[sql]

    def sqlChipMaker(self, sql, loc=0):
        sqlFragments = []

        def deepenSqlChip(stmt):
            # root
            if hasattr(stmt, "tokens"):
                for tok in stmt.tokens:
                    deepenSqlChip(tok)
            else:
                # leaf
                seg = str(stmt)
                if seg.strip() != "":
                    sqlFragments.append(seg)
                return

        format_sql = sqlparse.format(
            sql, reindent=True, keyword_case='upper', strip_comments=True, reindent_aligned=False)
        parsed = sqlparse.parse(format_sql)
        sql_stmt = parsed[loc]
        deepenSqlChip(sql_stmt)
        return sqlFragments

    # 获取sql动作
    #   @args：
    #      [sql] : str(sql)
    #      [loc] : index of sql, when there are multi-sqls in str(sql)
    #   @return：INSERT|SELECT|UPDATE|DELETE
    def getSqlAction(self, sql, loc=0):
        sqlSegs = self.sqlChipMaker(sql, loc)
        return sqlSegs[0].upper()

    # 合并sql字符串
    # options = {maxSize:int,combType:bool}
    def multiSqlParse(self, base_sql, args_list, logging, parseType, options={}):
        # 解析参数
        maxSize = VarGet(options.get('maxSize'),__cache__.max_sql_size)
        combType = options.get('combType')
        quotation = VarGet(options.get('quotation'),__cache__.quotation)
        #
        sqlCache = []
        transSqls = ''
        transLen = 0
        #
        for args in args_list:
            transSql = self.sqlParse(
                base_sql, args, logging, parseType=parseType, quotation=quotation)
            # 通过正则去掉首尾的分号 ;
            pattern = re.compile(r'(^;*)|(;*$)', re.M)
            transSql = re.sub(pattern, '', transSql)
            # 为每个sql语句添加分号
            transSql = f'{transSql};'
            # 缓存sql
            sqlCache.append(transSql)
            # 合并sql
            if combType == 'INSERT':
                '''
                    # 只适用 insert into ??? values ??? 语句
                '''
                # 分解sql
                sqlSegs = self.sqlChipMaker(transSql, 0)
                # values后面的部分
                sqlValuesStr = ''.join(sqlSegs[sqlSegs.index('VALUES')+1:])
                # 长度累加
                if transLen == 0:
                    transLen = len(transSql)
                else:
                    transLen += len(sqlValuesStr)
                if transLen < maxSize:
                    if transSqls == '':
                        transSqls = transSql
                    else:
                        # 截取VALUES后面的值，并且去分号，与前一个sql去分号合并添加逗号
                        transSqls = transSqls[:-1] + ',' + sqlValuesStr
            elif combType == 'UPDATE':
                pass
            elif combType == 'DELETE':
                pass
            elif combType == 'SELECT':
                pass
            elif combType == 'TRUNCATE':
                pass
            elif combType == None:
                '''
                    # 直接合并sql语句，泛用各类sql语句，包括嵌套、表关联、子查询等等
                '''
                transLen += len(transSql)
                if transLen < maxSize:
                    transSqls += transSql

        return Store({
            'sqls': sqlCache,
            'transSqls': transSqls,  # 字符串连接后的sql字符串
        })

    # sql模板参数转换
    #   @args：
    #      [sql] : str(sql)
    #      [tuple_args|dict_args] :
    #      [parseType] : 1 | 2 | 3 | 4 | 5 | 6
    #           parseType == 1: (default parse , no parse engine)
    #               sql : %s %s %s        ==>   model : (p1, p2, p3)
    #           parseType == 2: (StrParseEngine)
    #               sql : {0} {1} {0} {2} ==>   model : (p1, p2, p3)
    #                   {0} = p1 , {1} = p2 , {2} = p3
    #           parseType == 3: (DictParseEngine)
    #               sql :  col1=:c1, col2=:c2  ==>   model : {'c1':p1, 'c2':p2}
    #   @return：str(sql)
    # 注意： parseType = 4 | 5 | 6 转换方式同 1 | 2 | 3 , 只是对于非数值类型的值自动补充双引号

    def sqlParse(self, *args, parseType, quotation):
        if str(parseType) == "1":
            return self.TempParseEngine(*args)
        elif str(parseType) == "2":
            return self.StrParseEngine(*args)
        elif str(parseType) == "3":
            return self.DictParseEngine(*args)
        elif str(parseType) == "4":
            return self.TempParseEngineT(*args, quotation=quotation)
        elif str(parseType) == "5":
            return self.StrParseEngineT(*args, quotation=quotation)
        elif str(parseType) == "6":
            return self.DictParseEngineT(*args, quotation=quotation)
        else:
            return self.DictParseEngine(*args)

    def TempParseEngine(self, sql, tuple_args, logging):
        sqlSegments = sql.split('\n')
        for segIdx in range(len(sqlSegments)):
            commentIdx = sqlSegments[segIdx].find('--')
            if commentIdx >= 0:
                sqlSegments[segIdx] = sqlSegments[segIdx].replace(
                    sqlSegments[segIdx][commentIdx:], "")
        sql = ' '.join(sqlSegments)
        logging("DEBUG", FormatMsg('SQL', sql))
        sql = sql % tuple(map(lambda v: str(v), tuple_args))
        _sql_ = sql.strip()
        logging("DEBUG", FormatMsg('SQL', _sql_))
        return _sql_

    def StrParseEngine(self, sql, tuple_args, logging):
        sqlSegments = sql.split('\n')
        for segIdx in range(len(sqlSegments)):
            commentIdx = sqlSegments[segIdx].find('--')
            if commentIdx >= 0:
                sqlSegments[segIdx] = sqlSegments[segIdx].replace(
                    sqlSegments[segIdx][commentIdx:], "")
        sql = ' '.join(sqlSegments)
        logging("DEBUG", FormatMsg('SQL', sql))
        sql = sql.format(*tuple(map(lambda v: str(v), tuple_args)))
        _sql_ = sql.strip()
        logging("DEBUG", FormatMsg('SQL', _sql_))
        return _sql_

    def DictParseEngine(self, sql, dict_args, logging):
        sql = self.formatSql(sql)
        logging("DEBUG", FormatMsg('SQL', sql))
        sqlFragments = self.sqlChipMaker(sql)
        for key in dict_args:
            sqlFragments = self.listReplace(
                sqlFragments, f':{key}', str(dict_args[key]))
        sql = self.formatSql(' '.join(sqlFragments))
        _sql_ = sql.strip()
        logging("DEBUG", FormatMsg('SQL', _sql_))
        return _sql_

    def TempParseEngineT(self, sql, tuple_args, logging, quotation):
        sqlSegments = sql.split('\n')
        for segIdx in range(len(sqlSegments)):
            commentIdx = sqlSegments[segIdx].find('--')
            if commentIdx >= 0:
                sqlSegments[segIdx] = sqlSegments[segIdx].replace(
                    sqlSegments[segIdx][commentIdx:], "")
        sql = ' '.join(sqlSegments)
        logging("DEBUG", FormatMsg('SQL', sql))
        sql = sql % tuple(map(lambda v: str(v) if type(v) == int or type(
            v) == float else f'{quotation}{str(v)}{quotation}', tuple_args))
        _sql_ = sql.strip()
        logging("DEBUG", FormatMsg('SQL', _sql_))
        return _sql_

    def StrParseEngineT(self, sql, tuple_args, logging, quotation):
        sqlSegments = sql.split('\n')
        for segIdx in range(len(sqlSegments)):
            commentIdx = sqlSegments[segIdx].find('--')
            if commentIdx >= 0:
                sqlSegments[segIdx] = sqlSegments[segIdx].replace(
                    sqlSegments[segIdx][commentIdx:], "")
        sql = ' '.join(sqlSegments)
        logging("DEBUG", FormatMsg('SQL', sql))
        sql = sql.format(*tuple(map(lambda v: str(v) if type(v) ==
                                    int or type(v) == float else f'{quotation}{str(v)}{quotation}', tuple_args)))
        _sql_ = sql.strip()
        logging("DEBUG", FormatMsg('SQL', _sql_))
        return _sql_

    def DictParseEngineT(self, sql, dict_args, logging, quotation):
        sql = self.formatSql(sql)
        logging("DEBUG", FormatMsg('SQL', sql))
        sqlFragments = self.sqlChipMaker(sql)
        for key in dict_args:
            sqlFragments = self.listReplace(
                sqlFragments, f':{key}', str(dict_args[key]) if type(dict_args[key]) == int or type(dict_args[key]) == float else f'{quotation}{str(dict_args[key])}{quotation}')
        sql = self.formatSql(' '.join(sqlFragments))
        _sql_ = sql.strip()
        logging("DEBUG", FormatMsg('SQL', _sql_))
        return _sql_



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    模板工厂
#    解析模板指令
# Created : 2021.04.22
# Author : Chalk Yu
# ========================================
from functools import reduce
import re
from .screws import Store
'''
@model``
@receive``
@execute``
@repeat``
'''


class TemplateFactory():
    def __init__(self):
        self.keywords = [
            'model',
            'receive',
            'execute',
            'repeat'
        ]

    def composition(self, strContent, keywordIndex):
        expIdxArr = []
        # 表达式不存在
        if not self.keywords[keywordIndex]:
            return ([strContent], [])
        partern = re.compile(f'\@{self.keywords[keywordIndex]}\s*\`[^\`]*\`')
        expArr = partern.findall(strContent)  # 表达式数组
        expSpl = partern.split(strContent)  # 被表达式分割的字符串数组
        # 匹配不到表达式
        if len(expArr) == 0:
            return ([strContent], [])
        return (expArr, expSpl)

    # 碎片化

    def fragmentization(self, strContent, keywordIndex=0):
        expArr, expSpl = self.composition(strContent, keywordIndex)
        if len(expSpl) == 0:
            return expArr
        self.expCur = 0  # 表达式数组当前指针

        def callback(target, current):
            resArr = []
            if(type(target) == str):
                resArr = self.fragmentization(
                    target, keywordIndex+1) + [expArr[self.expCur]] + self.fragmentization(current, keywordIndex+1)
            else:
                resArr = target + [expArr[self.expCur]] + \
                    self.fragmentization(current, keywordIndex+1)
            self.expCur = self.expCur + 1
            return resArr
        return reduce(callback, expSpl)

    # 从指令函数中，拆解指令参数，如： @model`User` => User
    def getArg(self, strExp, directName):
        if f'@{directName}' in strExp and strExp.strip().index(f'@{directName}') == 0:
            arg = strExp.replace(f'@{directName}', '').strip()
            return arg[1:-1]

    # 判断是否是某个指令
    def checkIsDirective(self, strExp, directName):
        if f'@{directName}' in strExp and strExp.strip().index(f'@{directName}') == 0:
            arg = strExp.replace(f'@{directName}', '').strip()
            if arg[0] == '`' and arg[-1] == '`':
                return True
            else:
                return False
        else:
            return False

    '''
        @model指令，用于给长sql语句分段，如：
        
        -- 这条sql的模块名是user
        @model`user`
        select * from user;
        -- 这条sql的模块名是address
        @model`address`
        select * from address;
    '''

    def Model(self, fullSql):
        sqlChips = Store()
        sectionName = None
        sqlLines = fullSql.split('\n')
        for sqlLine in sqlLines:
            # 数据如： ['   ', '@model`user`', '   ']
            # fraArr = self.fragmentization(sqlLine)
            # if len(fraArr) == 3 and fraArr[0].strip() == "" and self.checkIsDirective(fraArr[1], 'model') and fraArr[2].strip() == "":
            # directive = fraArr[1]
            fraArr, splitStr = self.composition(sqlLine, 0)
            if len(splitStr) > 0 and ''.join(splitStr).strip() == "":
                directive = fraArr[0]
                sectionName = self.getArg(directive, 'model').strip()
                sqlChips[sectionName] = ""
            else:
                if sectionName != None:
                    sqlChips[sectionName] = sqlChips[sectionName] + \
                        '\n' + sqlLine
                    sqlChips[sectionName] = sqlChips[sectionName].strip()
        return sqlChips


templatePaser = TemplateFactory()



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    ORM Loader 外部接口加载器
# Created : 2020.10.22
# Author : Chalk Yu
# ========================================
from __future__ import absolute_import
from .extends import OrmLoader, SqlParser


#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

long_description = '''
============
Straw 数据管
============

.. image:: https://img.shields.io/badge/Beta-0.9.x-yellow
.. image:: https://img.shields.io/badge/License-MIT-green
.. image:: https://img.shields.io/badge/Python-%3E%3D%203.7-blue
.. image:: https://img.shields.io/badge/SQL%40Support-postgres%20%7C%20mysql-lightgrey

**简单的函数调用来处理数据库**

*Straw可以方便的在多个不同的数据库之间传输数据，只需要调用一个方法把数据取出来，再调用一个方法把数据插到另一个数据库中
他可以像MyBatis一样，分离sql层和逻辑层，但又不同于MyBatis的是你可以在脚本中使用它，你还可以同时连接多个数据库
如果你只想写一个简单的脚本，把爬到的数据插到数据中，Straw会非常简单*

`[How to start] <https://github.com/pskelecton/straw>`_

**MIT License**

*Copyright (c) 2021 Chalk Yu (Straw)*

*Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:*

*The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.*

*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.*
'''

# import os
# dir = os.path.dirname(os.path.abspath(__file__))
# with open(os.path.join(dir,'readme.rst'),encoding='utf-8') as f:
#     long_description = f.read()

setup(
    name='pstraw',
    version='0.9.7',
    description=('简单的函数调用来处理数据库'),
    long_description=long_description,
    author='Chalk Yu',
    author_email='yuequn3721@qq.com',
    maintainer='Chalk Yu',
    maintainer_email='yuequn3721@qq.com',
    license='MIT License',
    packages=find_packages(),
    include_package_data=True,
    platforms=["all"],
    url='https://github.com/pskelecton/straw',
    install_requires=[
        "sqlalchemy==1.3.22",
        "sqlparse==0.4.1",
        "pychalk==2.0.1",
        "promise==2.3",
        "psycopg2==2.8.6",
        "pymysql==0.9.3"
    ],
    classifiers=[
        'Programming Language :: Python :: 3.7'
    ],
)



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    兼容多数据库 Loader
# Created : 2020.10.22
# Author : Chalk Yu
# ========================================
from __future__ import absolute_import
from .extends import SqlParser, OrmLoader
import os
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, relationship, scoped_session
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.pool import NullPool
from ..tool import FormatMsg
from ..screws import Store


class loader(OrmLoader, SqlParser):
    def __init__(self):
        super().__init__()

    def connect(self, dbConf=None):
        dbAccess = Store({
            'DB_DRIVER': self.DB_DRIVER,
            'DB_USER': self.DB_USER,
            'DB_PASSWORD': self.DB_PASSWORD,
            'DB_HOST': self.DB_HOST,
            'DB_PORT': self.DB_PORT,
            'DB_DATABASE': self.DB_DATABASE,
            'ENCODING': self.ENCODING,
            'ALLOW_ROLLBACK': self.ALLOW_ROLLBACK,
            'AUTO_COMMIT': self.AUTO_COMMIT,
            'SQLALCHEMY_ARGS': self.SQLALCHEMY_ARGS
        })
        if dbConf != None:
            dbAccess = dbConf

        conn_str = ''
        if dbAccess.DB_DRIVER == "postgres":
            conn_str = 'postgresql+psycopg2://%s:%s@%s:%s/%s' % (
                dbAccess.DB_USER, dbAccess.DB_PASSWORD, dbAccess.DB_HOST, dbAccess.DB_PORT, dbAccess.DB_DATABASE)
        elif dbAccess.DB_DRIVER == "mysql":
            conn_str = 'mysql+pymysql://%s:%s@%s:%s/%s' % (
                dbAccess.DB_USER, dbAccess.DB_PASSWORD, dbAccess.DB_HOST, dbAccess.DB_PORT, dbAccess.DB_DATABASE)
        elif dbAccess.DB_DRIVER == "oracle":
            conn_str = 'oracle+cx_oracle://%s:%s@%s:%s/%s' % (
                dbAccess.DB_USER, dbAccess.DB_PASSWORD, dbAccess.DB_HOST, dbAccess.DB_PORT, dbAccess.DB_DATABASE)
        elif dbAccess.DB_DRIVER == "mssql":
            conn_str = 'mssql+pymssql://%s:%s@%s:%s/%s' % (
                dbAccess.DB_USER, dbAccess.DB_PASSWORD, dbAccess.DB_HOST, dbAccess.DB_PORT, dbAccess.DB_DATABASE)
        elif dbAccess.DB_DRIVER == "sqlite":
            conn_str = 'sqlite:///%s' % dbAccess.SQLITE_PATH
        else:
            raise Exception(
                FormatMsg("DB Driver must be one of postgres/mysql."))
        # 连接数据库
        if conn_str != '':
            # sqlalchemy扩展参数
            engine = None
            if dbAccess.SQLALCHEMY_ARGS and type(dbAccess.SQLALCHEMY_ARGS.create_engine) == dict:
                if dbAccess.SQLALCHEMY_ARGS.create_engine['poolclass'] == None:
                    dbAccess.SQLALCHEMY_ARGS.create_engine['poolclass'] = NullPool
                if dbAccess.SQLALCHEMY_ARGS.create_engine['encoding'] == None:
                    dbAccess.SQLALCHEMY_ARGS.create_engine['encoding'] = dbAccess.ENCODING
                engine = create_engine(
                    conn_str, **dbAccess.SQLALCHEMY_ARGS.create_engine)
            else:
                engine = create_engine(
                    conn_str, poolclass=NullPool, encoding=dbAccess.ENCODING)
            #
            sessionFacory = None
            if dbAccess.SQLALCHEMY_ARGS and type(dbAccess.SQLALCHEMY_ARGS.sessionmaker) == dict:
                if dbAccess.SQLALCHEMY_ARGS.sessionmaker['autoflush'] == None:
                    dbAccess.SQLALCHEMY_ARGS.sessionmaker['autoflush'] = True
                sessionFacory = sessionmaker(
                    bind=engine, **dbAccess.SQLALCHEMY_ARGS.sessionmaker)
            else:
                sessionFacory = sessionmaker(bind=engine, autoflush=True)
            #
            __connection__ = None
            if dbAccess.SQLALCHEMY_ARGS and type(dbAccess.SQLALCHEMY_ARGS.scoped_session) == dict:
                __connection__ = scoped_session(
                    sessionFacory, **dbAccess.SQLALCHEMY_ARGS.scoped_session)
            else:
                __connection__ = scoped_session(sessionFacory)
            #
            return __connection__

    def execute(self, conn, sqlStr, sqlAction):
        # self._SqlAction_ = sqlAction
        try:
            cur = conn.execute(sqlStr)
            self.logging("DEBUG", FormatMsg("SQL execute success."))
            return cur
        except Exception as exc:
            self.logging("ERROR", FormatMsg("SQL execute error."))
            raise Exception("%s: %s" % (exc.__class__.__name__, exc))

    def close(self, conn):
        conn.close()

    def commit(self, conn):
        conn.commit()

    def rollback(self, conn):
        conn.rollback()

    def inject(self, conn, sqlAction, cursor, bean, resultCreater):
        if(cursor == None):
            raise Exception(
                FormatMsg("The 'cursor' returned by the 'execute' function cannot be 'None'."))
        if sqlAction == 'SELECT':
            res = resultCreater(list)(cursor=cursor, count=None)
            rowcount = 0
            for record in cursor:
                res.append(bean(*record))
                rowcount += 1
            res.count = rowcount
            return res
        elif sqlAction == 'INSERT':
            res = resultCreater()(cursor=cursor, count=None)
            return res
        elif sqlAction == 'UPDATE':
            res = resultCreater()(cursor=cursor, count=None)
            return res
        elif sqlAction == 'DELETE':
            res = resultCreater()(cursor=cursor, count=None)
            return res
        elif sqlAction == 'TRUNCATE':
            res = resultCreater()(cursor=cursor, count=None)
            return res
        else:
            raise Exception(FormatMsg(
                f'The sql action must be one of SELECT, INSERT, UPDATE, DELETE, TRUNCATE.'))



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    工具类
#    反射类、数据仓库类、路径加工、缓存
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from promise import Promise
import os
import inspect
from dataclasses import dataclass

# 字典反射对象使用案例
# ConfStore = Store({})
# ConfStore.a = {'xxx': Store({'yyy': 123})}
# ConfStore.b = {'xxx': {'yyy': 123}}

# 配置对象，控制对象只读


class ConfStore():
    def __init__(self, **kwargs):
        self.__ConfStore__ = Store(kwargs)

    def __repr__(self):
        return str(self.__ConfStore__)

    def __str__(self):
        return str(self.__ConfStore__)

    def __getattr__(self, name):
        return self.__ConfStore__.get(name)

    def create(self, name, value):
        if self.__ConfStore__.get(name) == None:
            self.__ConfStore__[name] = value
            return True
        else:
            return False

    def modify(self, name, value):
        if self.__ConfStore__.get(name) != None:
            self.__ConfStore__[name] = value
            return True
        else:
            return False

    def remove(self, name):
        del self.__ConfStore__[name]


# 路径加工类
class PathPlant():
    def __init__(self, *args, **kwargs):
        super().__init__()
        # 接收一个model名的参数
        self.MODEL_FOLDER_NAME = kwargs.get('MODEL_FOLDER_NAME') if not hasattr(
            self, "MODEL_FOLDER_NAME") else getattr(self, "MODEL_FOLDER_NAME")

    # 转换绝对路径，纯静态方法，外部直接访问
    # 规则转换: @/xxx/ 则只取相对路径 , 没有@/则转换绝对路径
    @staticmethod
    def transAbspath(path):
        if type(path) == str:
            if path[:2] == '@/':
                # 相对路径转换
                # return os.path.normpath(path[2:])
                # 绝对路径转换
                return os.path.realpath(path[2:])
            else:
                # 绝对路径转换
                return os.path.abspath(path)
        else:
            return None

    # 获取模型绝对路径
    @classmethod
    def getModelPath(cls, model_func):
        # 获取模型绝对路径
        model_path = os.path.abspath(inspect.getfile(model_func))
        return model_path

    # 初始化文件夹，静态+对象方法，可直接访问，可继承访问
    @classmethod
    def splitFolder(cls, path, modelStr, includeModel=True):
        # 文件或目录检测,解决path最后带\\或/的情况
        tempath = os.path.split(path)
        if tempath[1] == '':
            path = tempath[0]
        # modelStr前的路径
        head = path
        # modelStr后的路径
        tail = ""
        #
        skip = False
        while not skip:
            sp = os.path.split(head)
            # 匹配到或者都没有匹配到则跳出
            if sp[1] == modelStr or sp[0] == "":
                skip = True
            # 初始赋值
            if tail == "":
                tail = sp[1]
            else:
                if sp[1] == modelStr and includeModel == False:
                    # 不组装modelStr
                    pass
                else:
                    # 每次循环都合并一下路径
                    tail = os.path.join(sp[1], tail)
            head = sp[0]
        return (head, tail)

    @classmethod
    def initFolder(cls, path):
        # 判断路径是否存在，不存在则创建文件夹
        if not os.path.exists(path):
            # os.mkdir(path, mode=0o777)
            os.makedirs(path, mode=0o777, exist_ok=False)

    # 深度遍历文件，path一定要是绝对路径，静态+对象方法，可直接访问，可继承访问
    @classmethod
    def deepenFolder(cls, path):
        _file_list = list()
        _dep_file_dict = dict()

        # 结构化文件夹
        def traverseFile(dir_path):
            if os.path.exists(dir_path) and os.path.isdir(dir_path):
                file_list = os.listdir(dir_path)
                dep_file_dict = dict()
                for file_name in file_list:
                    file_path = os.path.join(dir_path, file_name)
                    if os.path.isdir(file_path):
                        dep_file_dict[file_name] = traverseFile(file_path)
                    else:
                        _file_list.append(file_path)
                        dep_file_dict[file_name] = file_path
                return dep_file_dict
            else:
                raise Exception

        # 生成文件结构化对象
        _dep_file_dict = traverseFile(os.path.abspath(path))

        return Store({'FILE_STRUCTURE': _dep_file_dict, 'PATH_LIST': _file_list})


# 字典反射成对象
class Store(dict):
    # 缓冲数据
    __cache_dict = None

    def __init__(self, iterable=None):
        iterable = iterable or {}
        self.__cache_dict = dict(iterable)
        super().__init__(iterable)
        self.__reflect__(iterable)

    # 构建时反射值
    def __reflect__(self, _dict):
        for key in _dict:
            if type(_dict[key]) == dict:
                self.__dict__[key] = Store(_dict[key])
            else:
                self.__dict__[key] = _dict[key]

    # 更新时同步反射
    def __updvalue__(self, key, value):
        # 判断旧值不等于新值的情况
        if str(self.__cache_dict.get(key)) != str(value):
            # 更新缓冲数据
            self.__cache_dict[key] = value
            # 引用赋值
            if type(value) == dict:
                self.__dict__[key] = Store(value)
            else:
                self.__dict__[key] = value
            # 赋值
            super().__setitem__(key, value)

    # 对象属性赋值拦截
    def __setattr__(self, key, value):
        if self.__cache_dict != None:
            self.__updvalue__(key, value)
        else:
            self.__dict__[key] = value

    # 字典属性赋值拦截
    def __setitem__(self, key, value):
        if self.__cache_dict != None:
            self.__updvalue__(key, value)

    # 字典取值拦截
    def __getitem__(self, key):
        try:
            return self.__dict__[key]
        except Exception as exc:
            return None

    # 解除属性引用
    def __delattr__(self, key):
        try:
            del self.__dict__[key]
            super().__delitem__(key)
        except Exception as exc:
            pass

    # 解除字典属性引用
    def __delitem__(self, key):
        try:
            del self.__dict__[key]
            super().__delitem__(key)
        except Exception as exc:
            pass



#!/usr/local/bin/python3.6
# -*- coding:utf-8 -*-
# ========================================
# Description :
#    Bean工厂
#    默认Bean接口，Bean的加载器
# Created : 2020.10.14
# Author : Chalk Yu
# ========================================
from dataclasses import dataclass
import sqlparse


class Bean():
    def __init__(self):
        super().__init__()


class BeanFactory():
    def __init__(self):
        super().__init__()

    def createResultClass(self, *classType):
        class ResultBean(*classType):
            def __init__(self, *args, **kwargs):
                if len(classType) > 0:
                    super().__init__(*args)
                self.__cursor = kwargs.get('cursor') or None
                self.__count = kwargs.get('count') or None

            @property
            def cursor(self):
                return self.__cursor

            @cursor.setter
            def cursor(self, _cursor):
                # 有值后即被锁死
                if self.__cursor == None:
                    self.__cursor = _cursor

            @property
            def count(self):
                return self.__count

            @count.setter
            def count(self, _count):
                # 有值后即被锁死
                if self.__count == None:
                    self.__count = _count
        return ResultBean

    def _inject_(self, fn):
        def __inject__(*args, **kwargs):
            kwargs['__bean_factory__'] = self
            return fn(*args, **kwargs)
        return __inject__


bf = BeanFactory()
